Coordinates

Struct Coordinates 

pub struct Coordinates(/* private fields */);
A valid pair of geographic coordinates.

See https://en.wikipedia.org/wiki/Geographic_coordinate_system

Implementations

Source

impl Coordinates

pub const fn new(lat: f64, lon: f64) -> Option<Self>

Validate a pair of latitude / longitude.

Return None if values are out of range (abs(lat) > 90 or abs(lon) > 180).

pub fn event_time( &self, date: NaiveDate, event: TimeEvent, ) -> Option<DateTime<Utc>>

Get the time for a sun event at a given date.

pub fn lat(&self) -> f64

Get latitude component.

pub fn lon(&self) -> f64

Get longitude component.