Schedule

Struct Schedule 

pub struct Schedule { /* private fields */ }
Describe a full schedule for a day, keeping track of open, closed and unknown periods.

It can be turned into an iterator which will yield consecutive ranges of different states, with no holes or overlapping.

Implementations

Source

impl Schedule

pub fn new() -> Self

Creates a new empty schedule, which represents an always closed period.

use opening_hours::schedule::Schedule;

assert!(Schedule::new().is_empty());
pub fn from_ranges( ranges: impl IntoIterator<Item = Range<ExtendedTime>>, kind: RuleKind, comments: &UniqueSortedVec<Arc<str>>, ) -> Self

Create a new schedule from a list of ranges of same kind and comment.

use opening_hours::schedule::Schedule;
use opening_hours_syntax::{ExtendedTime, RuleKind};

let sch1 = Schedule::from_ranges(
    [
        ExtendedTime::new(10, 0).unwrap()..ExtendedTime::new(14, 0).unwrap(),
        ExtendedTime::new(12, 0).unwrap()..ExtendedTime::new(16, 0).unwrap(),
    ],
    RuleKind::Open,
    &Default::default(),
);

let sch2 = Schedule::from_ranges(
    [ExtendedTime::new(10, 0).unwrap()..ExtendedTime::new(16, 0).unwrap()],
    RuleKind::Open,
    &Default::default(),
);

assert_eq!(sch1, sch2);
pub fn is_empty(&self) -> bool

Check if a schedule is empty.

use opening_hours::schedule::Schedule;

assert!(Schedule::new().is_empty());
pub fn addition(self, other: Self) -> Self

Merge two schedules together.