#[non_exhaustive]pub struct DateTimeRange<D = NaiveDateTime> {
pub range: Range<D>,
pub kind: RuleKind,
pub comment: Arc<str>,
}Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.range: Range<D>kind: RuleKindcomment: Arc<str>Implementations
Sourceimpl<D> DateTimeRange<D>
impl<D> DateTimeRange<D>
Sourcepub fn into_state(self) -> (RuleKind, Arc<str>)
pub fn into_state(self) -> (RuleKind, Arc<str>)
Extract the kind and comment from the range, which are the values that define current state of an expression.