Structs
- Coordinates
- A valid pair of geographic coordinates.
- NoLocation
- TzLocation
- Time zone is specified and coordinates can optionally be specified for accurate sun events.
Enums
- Country
- An enum for selecting a country.
Traits
- Localize
- Specifies how dates should be localized while evaluating opening hours. No localisation is available by default but this can be used to specify a timezone and coordinates (which affect sun events).